@@ -1,8 +1,10 @@
-from typing import Dict, Any, List, Union
+from typing import Dict, Any, List, Union, Optional, Literal
 from dataclasses import is_dataclass, fields
 from collections import defaultdict
+from enum import Enum
 
 from .route_def import RouteDef
+from .auth import AuthBackend
 
 _pytype_to_schema_type_lookup = {
 	int: 'integer',
@@ -16,12 +18,13 @@ def pytype_to_schema(py_type: type) -> Dict[str, Any]:
 	origin_type = getattr(py_type, '__origin__', None)
 	generic_types = getattr(py_type, '__args__', None)
 
-	if is_dataclass(py_type):
-		return {
-			'type': 'object',
-			'properties': { f.name: pytype_to_schema(f.type) for f in fields(py_type) },
-		}
-	elif origin_type is not None:
+	if not origin_type:
+		if is_dataclass(py_type):
+			return {
+				'type': 'object',
+				'properties': { f.name: pytype_to_schema(f.type) for f in fields(py_type) },
+			}
+	else:
 		if origin_type == list:
 			return {
 				'type': 'array',
@@ -39,13 +42,18 @@ def pytype_to_schema(py_type: type) -> Dict[str, Any]:
 			return {
 				'oneOf': [ pytype_to_schema(t) for t in generic_types ],
 			}
+		elif origin_type == Literal:
+			return {
+				'type': _pytype_to_schema_type_lookup[type(generic_types[0])],
+				'enum': generic_types,
+			}
 
 	schema_type = _pytype_to_schema_type_lookup[py_type]
 	return {
 		'type': schema_type,
 	}
 
-def build_schemas(route_defs: List[RouteDef], api_name: str) -> Dict[str, Any]:
+def build_schemas(route_defs: List[RouteDef], api_name: str, auth_backend: Optional[AuthBackend]) -> Dict[str, Any]:
 	paths: Dict[str, Any] = defaultdict(dict)
 	for route_def in route_defs:
 		spec: Any = {
@@ -83,13 +91,23 @@ def build_schemas(route_defs: List[RouteDef], api_name: str) -> Dict[str, Any]:
 
 		paths[route_def.path][route_def.http_method] = spec
 
-	openapi_spec = {
+	openapi_spec: Dict[str, Any] = {
 		'openapi': '3.0.0',
 		'info': {
 			'title': api_name,
 			'version': '0.1',
 		},
 		'paths': paths,
+		'components': {},
 	}
 
+	if auth_backend:
+		auth_name, spec = auth_backend.openapi_spec()
+		openapi_spec['components']['securitySchemes'] = {
+			auth_name: spec,
+		}
+		openapi_spec['security'] = [
+			{ auth_name: [] },
+		]
+
 	return openapi_spec
